<?php
//
// NRDP Plugin - NRDS Updater
// This plugin allows NRDS to send configs and plugins to anything that
// can use the NRDS API.
//
// Written by: Scott Wilkerson
// Copyright (c) 2012-2017 Nagios Enterprises, LLC.
// 

require_once(dirname(__FILE__).'/../../config.inc.php');
require_once(dirname(__FILE__).'/../../includes/utils.inc.php');

global $cfg;
$cfg["config_dir"] = "/usr/local/nrdp/configs";
$cfg["plugin_root"] = "/usr/local/nrdp/plugins";

if (!array_key_exists("tmp_dir", $cfg)) {
    $cfg["tmp_dir"] = "/tmp";
}

register_callback(CALLBACK_PROCESS_REQUEST, 'nrdsconfigupdate_process_request');

function nrdsconfigupdate_process_request($cbtype, $args)
{
    $cmd = grab_array_var($args, "cmd");

    switch ($cmd)
    {
        // Check data
        case "updatenrds":
            nrds_config_update_check();
            break;

        case "getconfig":
            nrds_get_config();
            break;

        case "getplugin":
            nrds_get_plugin();
            break;

        case "nrdsgetclient":
            nrds_get_client_install();
            break;

        // Something else we don't handle...
        default:
            break;
    }
}

function nrds_get_client_install()
{
    global $cfg;
    global $request;
    $configname = grab_request_var("configname");
    $token = grab_request_var("token");

    $config_file = $cfg["config_dir"]."/".$configname.".cfg";

    // Figure out the operating system for this configuration
    $handle = @fopen($cfg["config_dir"]."/".$configname.".cfg", "r");
    if ($handle) {
        while (($buffer = fgets($handle, 4096)) !== false) {
            if (substr($buffer,0,9) == "CONFIG_OS") {
                $os_line = explode("=", $buffer);
                $os = trim($os_line[1], "\" \t\n\r\0\x0B");
            }
        }
        fclose($handle);
    }

    if (substr($os, 0 ,7) == "Windows") {

        if ($os == "Windows (32-bit)") {
            $target_bld_script = "NRDS_Win_32.nsi";
            $client_install_file = "NRDS_Win_32.exe";
        } else if ($os == "Windows (64-bit)") {
            $target_bld_script = "NRDS_Win_64.nsi";
            $client_install_file = "NRDS_Win_64.exe";
        }

        $nsis_root = "/usr/local/nsis/bin/";
        $nsis_bin = $nsis_root."makensis";

        $nrds_win_root = "/usr/local/nrdp/win_clients/";
        $nsis_bld_dir = $nrds_win_root."build_files/";
        $nsis_tmp = $nrds_win_root."tmp/";

        $nsis_bld_path = $nsis_tmp.$target_bld_script;

        $script = "rm -rf $nsis_tmp*;";
        $script .= "cp -fr $nsis_bld_dir* $nsis_tmp; ";
        $script .= $nrds_win_root."nrds_win_taylor.py";
        $script .= " $nsis_tmp";
        $script .= " $target_bld_script";
        $script .= " ".escapeshellarg("$configname".".cfg");
        $script .= " ".escapeshellarg($token);
        $script .= $nsis_bin." ".$nsis_bld_path.";";

        exec($script);
        $client_install_path = $nsis_tmp.$client_install_file;

    } else {

        $rand = uniqid();
        $nrds_config_tmp_dir = $cfg["tmp_dir"]."/nrdsconfig_$rand";
        $client_install_file = "$configname.tar.gz";
        $client_install_path = "$nrds_config_tmp_dir/$client_install_file";
        $script = "mkdir -p $nrds_config_tmp_dir;";
        $script .= "cp -fr /usr/local/nrdp/clients $nrds_config_tmp_dir;";
        $script .= "chmod +x $nrds_config_tmp_dir/clients/installnrds;";
        $script .= "cp -f ".escapeshellarg($config_file)." $nrds_config_tmp_dir/clients/nrds/nrds.cfg;";
        $script .= "cd $nrds_config_tmp_dir;";
        $script .= "tar czfp ".escapeshellarg("$nrds_config_tmp_dir/$configname.tar.gz")." clients;";
        $script .= "rm -rf $nrds_config_tmp_dir/clients;";
        $rslt = system($script, $retval);

    }

    if (file_exists($client_install_path)) {
        header("Content-Type: application/octet-stream");
        header("Content-Disposition: attachment; filename=\"$client_install_file\"");
        header("Content-Length: ".filesize($client_install_path));
        passthru("cat ".escapeshellarg($client_install_path), $err);
        if (substr($os, 0, 7) != "Windows") {
            unlink($client_install_path);
            rmdir($nrds_config_tmp_dir);
        }
        exit();
    } else {
        header("HTTP/1.0 404 Not Found");
        exit;
    }
}

// This function grabs the config for delivery
function nrds_get_config()
{
    global $cfg;
    global $request;

    $configname = grab_request_var("configname");
    $os = grab_request_var("os", "");
    $arch = grab_request_var("arch", "");
    $os_ver = grab_request_var("os_ver", "");
    $hostname = grab_request_var("hostname", "");

    $config = nrds_get_config_as_string($configname, $os);
    if ($config != "") {
        print $config;
        exit;
    } else {
        header("HTTP/1.0 404 Not Found");
        exit;
    }
}

function nrds_get_plugin()
{
    global $cfg;
    global $request;
    $plugin = grab_request_var("plugin");
    $os = grab_request_var("os", "");
    $arch = grab_request_var("arch", "");
    $os_ver = grab_request_var("os_ver", "");
    $hostname = grab_request_var("hostname", "");

    /* TODO
     * I think we need some logic to group OS's
     * Current folders are:
     * AIX, Linux, Darwin (OSX), Solaris, Windows, Generic
     */
    if (fnmatch("i[3-6]86", $arch)) $arch = "i386";

    // These are the potential plugin paths, they will be searched in order
    $possible_plugin_paths = array(
        "{$cfg['plugin_root']}/$os/$arch/$os_ver/$plugin",
        "{$cfg['plugin_root']}/$os/$arch/$plugin",
        "{$cfg['plugin_root']}/$os/$plugin",
        "{$cfg['plugin_root']}/Generic/$arch/$plugin",
        "{$cfg['plugin_root']}/Generic/$plugin",
        "/usr/local/nagios/libexec/$plugin"
    );

    foreach ($possible_plugin_paths as $possible_plugin_paths) {
        if (file_exists($possible_plugin_paths)) {
            header("Content-Type: application/octet-stream");
            header("Content-Disposition: attachment; filename=\"$plugin\"");
            header("Content-Length: ".filesize($possible_plugin_paths));
            passthru("cat $possible_plugin_paths", $err);
            exit();
        }
    }

    header("HTTP/1.0 404 Not Found");
    exit;
}

function nrds_config_update_check()
{
    global $cfg;
    global $request;

    $debug = false;

    if ($debug) {
        echo "REQUEST:<BR>";
        print_r($request);
        echo "<BR>";
    }

    $configname = grab_request_var("configname", "");
    $version = grab_request_var("version", "");
    $os = grab_request_var("os", "");
    $arch = grab_request_var("arch", "");
    $os_ver = grab_request_var("os_ver", "");
    $hostname = grab_request_var("hostname", "");

    if (empty($configname) && empty($version)) {
        // The previous method passed the configuration name and version embedded
        // in XML. The code in this section handles this deprecated method.

        // Check results are passed as XML data
        $xmldata = grab_request_var("XMLDATA");

        if ($debug) {
            echo "XMLDATA:<BR>";
            print_r($xmldata);
            echo "<BR>";
        }

        // Make sure we have data
        if (!have_value($xmldata))
            handle_api_error(ERROR_NO_DATA);

        // Convert to XML
        $xml = @simplexml_load_string($xmldata);
        if (!$xml) {
            print_r(libxml_get_errors());
            handle_api_error(ERROR_BAD_XML);
        }

        if ($debug) {
            echo "OUR XML:<BR>";
            print_r($xml);
            echo "<BR>";
        }

        // Process each result
        foreach ($xml->config as $cr) {
            $configname = strval($cr->name);
            $version = doubleval($cr->version);
        }
    }

    // Setup defaults
    $status = 0;
    $message = "OK";
    $output = "";
    $config = "";

    // Check config logic
    if (!is_dir($cfg["config_dir"])) mkdir($cfg["config_dir"], 755, true);
    if (!file_exists($cfg["config_dir"]."/".$configname.".cfg")) {
        $status = 2;
        $message = "We do not have that configuration";
    } else {
        $config = nrds_get_config_as_string($configname, $os);
        $api_version = nrds_get_api_version($config);
        if ($version < $api_version) {
            $status = 1;
            $message = "Version ".$api_version." available";
        }
    }



    output_api_header();

    echo "<result>\n";
    echo "  <status>".$status."</status>\n";
    echo "  <message>".$message."</message>\n";
    echo "    <meta>\n";
    echo "       <output>".$output."</output>\n";
    echo "    </meta>\n";
    echo "  <config>\n";
    echo $config;
    echo "  </config>\n";
    echo "</result>\n";

    exit();
}

function nrds_get_config_as_string($configname, $os)
{
    global $cfg;

    $configfile = $cfg["config_dir"]."/".$configname.".cfg";
    $configcontents = "";

    if (file_exists($configfile)) {
        if ($os == "Windows") {
            $services_printed = 0;
            $configcontents .= "[settings]\r\n";
            $handle = @fopen( $configfile, "r");
            if ($handle) {
                while (($buffer = fgets($handle, 4096)) !== false) {
                    if ((substr( $buffer, 0, 7) == "command") and (0 == $services_printed)) {
                        $configcontents .= "[services]\r\n";
                        $services_printed = 1;
                    }
                    $configcontents .= trim($buffer) . "\r\n";
                }
                $configcontents .= "[extensions]\r\n";
                $configcontents .= 'vbs = cscript //nologo $CHECK_COMMAND$'."\r\n";
                $configcontents .= 'ps1 = powershell -ExecutionPolicy Bypass -File $CHECK_COMMAND$'."\r\n";
                fclose($handle);
            }
        } else {
            $configcontents = file_get_contents($configfile);
        }
        return $configcontents;
    } else {
        return "";
    }
}

function nrds_get_api_version($configcontents)
{
    $api_version = -1;
    preg_match( '/CONFIG_VERSION\s*=\s*.*\n/', $configcontents, $matches);
    if (count($matches) == 1) {
        $version_line = explode("=", $matches[ 0]);
        $api_version = trim($version_line[1], "\" \t\n\r\0\x0B");
    }
    return $api_version;
}
