!include "MUI.nsh"
!include "nsDialogs.nsh"
!include FileFunc.nsh
!insertmacro GetParameters
!insertmacro GetOptions

!define UNINST_KEY \
  "Software\Microsoft\Windows\CurrentVersion\Uninstall\NRDS_Win"
; ...
!define MULTIUSER_INSTALLMODE_COMMANDLINE
; ...
!include "MultiUser.nsh"

!define CONFIG_INI "$INSTDIR\Config.ini"

;The name of the installer
Name "NRDS_Win Installer"

;The file to write
OutFile "NRDS_Win_32.exe"
 
;The default installation directory
InstallDir $PROGRAMFILES\Nagios\NRDS_Win

;request admin execution
RequestExecutionLevel admin

;Settings
ShowInstDetails show

LoadLanguageFile "${NSISDIR}\Contrib\Language files\English.nlf"

;Order of pages
!define MUI_LANGUAGEFILE_DEFAULT "ENGLISH"
LangString MUI_INNERTEXT_LICENSE_BOTTOM "ENGLISH" "Nagios Software License 1.3"
LangString MUI_TEXT_LICENSE_TITLE "ENGLISH" "Nagios Enterprises LLC"
LangString MUI_TEXT_LICENSE_SUBTITLE "ENGLISH" "NRDS for Windows R4"
LangString MUI_INNERTEXT_LICENSE_TOP "ENGLISH" "Software License Agreement"
!insertmacro MUI_PAGE_LICENSE "NagiosSoftwareLicense.txt"
Page custom SetCustom "" ": Token and NRDP Address"
Page components
Page directory
Page instfiles

Function .onInit

  InitPluginsDir
	!insertmacro INSTALLOPTIONS_EXTRACT "field.ini"
	
	${GetParameters} $R0
	${GetParameters} $R1
	${GetParameters} $R2
	${GetParameters} $R3
	${GetParameters} $R4
	
  ClearErrors
  ${GetOptions} $R0 /NRDP= $0
	${GetOptions} $R1 /TOKEN= $1
	${GetOptions} $R2 /CONFIG= $2
	${GetOptions} $R3 /HOST= $3
	${GetOptions} $R4 /INTERVAL= $4

FunctionEnd


Function SetCustom

  ;Display the InstallOptions dialog
	InstallOptions::dialog "$PLUGINSDIR\field.ini"
	!define CHK_PROXYSETTINGS "Field 6"
	!insertmacro INSTALLOPTIONS_READ $0 "field.ini" "Field 1" "State"
	!insertmacro INSTALLOPTIONS_READ $1 "field.ini" "Field 2" "State"
	!insertmacro INSTALLOPTIONS_READ $2 "field.ini" "Field 3" "State"
	!insertmacro INSTALLOPTIONS_READ $3 "field.ini" "Field 4" "State"
	!insertmacro INSTALLOPTIONS_READ $4 "field.ini" "Field 5" "State"

FunctionEnd


Section "Create Config.ini"
	
	SetOutPath $INSTDIR

	File /r ..\NRDS_win_source\*.*

	WriteINIStr $INSTDIR\Config.ini settings "CONFIG_VERSION" "0"
	WriteINIStr $INSTDIR\Config.ini settings "CONFIG_NAME" "$2"
	WriteINIStr $INSTDIR\Config.ini settings "URL" "$0"
	WriteINIStr $INSTDIR\Config.ini settings "TOKEN" "$1"
	WriteINIStr $INSTDIR\Config.ini settings "PLUGIN_DIR" "$INSTDIR\plugins"
	WriteINIStr $INSTDIR\Config.ini settings "UPDATE_CONFIG" "1"
	WriteINIStr $INSTDIR\Config.ini settings "UPDATE_PLUGINS" "1"
	WriteINIStr $INSTDIR\Config.ini settings "LOG_FILE" "$INSTDIR\logs\NRDS_debug.log"
	
	WriteINIStr $INSTDIR\Host.ini settings "HOSTNAME" "$3"
	WriteINIStr $INSTDIR\Host.ini settings "CONFIG" "config.ini"

SectionEnd


Section ""
  ; ...
 
  WriteRegStr SHCTX "${UNINST_KEY}" "DisplayName" "NRDS_Win"
  WriteRegStr SHCTX "${UNINST_KEY}" "UninstallString" \
    "$\"$INSTDIR\uninstall.exe$\" /$MultiUser.InstallMode"
  WriteRegStr SHCTX "${UNINST_KEY}" "QuietUninstallString" \
    "$\"$INSTDIR\uninstall.exe$\" /$MultiUser.InstallMode /S"
 
  WriteUninstaller $INSTDIR\uninstall.exe
 
  ; ...
SectionEnd

Section "Uninstall"
	ReadEnvStr $9 COMSPEC
	
	ReadIniStr $8 $INSTDIR\Host.ini settings "HOSTNAME"
	
	nsExec::Exec '$9 /c "$INSTDIR\install\NRDS_Win_Remove.bat" $8'

	RMDir /r "$INSTDIR"
  DeleteRegKey SHCTX "${UNINST_KEY}"
	Delete "$INSTDIR\uninstall.exe"
	
SectionEnd
	
Section "Create Scheduled Task"
			
	nsisos::osversion
	StrCpy $R0 $0
	StrCpy $R1 $1
	
	# Get the OS version
	${if} $R0 == '5'
		nsExec::Exec /TIMEOUT=20000 '"$SYSDIR\cscript.exe" "$INSTDIR\install\NRDS_Win_Schedule.vbs" "$3" $4 "$INSTDIR"'
	${EndIf}
		
	${if} $R0 == '6'
		nsExec::Exec /TIMEOUT=20000 '"$SYSDIR\cscript.exe" "$INSTDIR\install\NRDS_Win_Schedule08.vbs" "$3" $4 "$INSTDIR"'
	${EndIf}

SectionEnd