#!/usr/bin/env python
import re
import sys
import logging

import logging

# create logger
logger = logging.getLogger('NRDS')
logger.setLevel(logging.DEBUG)

# create console handler and set level to debug
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# add formatter to ch
ch.setFormatter(formatter)

# add ch to logger
logger.addHandler(ch)

# 'application' code
logger.info('invoking NRDS Taylor')
logger.debug('entering debug mode')

def create_env(config):
    path = "/usr/local/nrdp/configs/"
    config_name = (re.split("\.",config))[0]
    full_path = path + config
    req_values = ['URL','TOKEN']
    return_val = {'CONFIG_NAME' : config_name}
    
    return req_values, return_val, full_path
    
def parse_nrds_config(file_to_open, req_values, return_val):
    f = open(file_to_open,'r')
    contents = f.read()
    split = re.split("\n", contents)
    a = [re.split("=", item) for item in split]
    
    for i in a:
        if i[0] in req_values:
            return_val.update({i[0]:i[1]})
    
    return return_val

def remove_silent_arge(full_path):
    logger.debug('remove silent install arguments')
    f = open(full_path, 'r+')
    logger.debug( "working file =" + full_path)
    contents = f.read()
    contents = re.sub(r'.\{GetOptions\} \$R\w+ \/\w+= \$\w+','',contents)
    f.seek(0)
    f.write(contents)
    f.truncate()
    f.close()

def parse_field_ini(key_items, build_path):
    field_path = build_path + "field.ini"
    f = open(field_path, 'r+')
    contents = f.read()
    url = key_items['URL']
    token = key_items['TOKEN']
    config_name = key_items['CONFIG_NAME']
    contents = re.sub(r'myUrl',url,contents)
    contents = re.sub(r'myToken',token,contents)
    contents = re.sub(r'myConfig',config_name,contents)
    print contents
    f.seek(0)
    f.write(contents)
    f.close()
    
def parse_nsi(nsi_file,repl,var_number,new,build_path):
    #~ path = "/usr/local/nsis/build_scripts/"
    full_path = build_path + nsi_file
    f = open(full_path, 'r+')
    contents = f.read()
    
    replace_exp = '''('''+ repl + '''" ")(.''' + var_number + ''')'''
    sub_match = repl + ".*" + var_number
    
    m = re.search(replace_exp,contents)
    
    if m and m.groups() > 0:
        test = m.group(1) + new
        contents = re.sub(sub_match,test,contents)
        f.seek(0)
        f.write(contents)
    f.close()
    
def main(args):
    build_path = args[1]
    target_script = args[2]
    config_name = args[3]
    
    req_values, return_val, full_path = create_env(config_name)
        
    remove_silent_arge(build_path + target_script)
    key_items = parse_nrds_config(full_path, req_values, return_val)
    parse_field_ini(key_items, build_path)
    parse_nsi(target_script,"CONFIG_NAME","2",key_items['CONFIG_NAME'],build_path)
    parse_nsi(target_script,"TOKEN","1",key_items['TOKEN'],build_path)
    parse_nsi(target_script,"URL","0",key_items['URL'],build_path)

main(sys.argv)
